package com.xxfc.rv;

import android.content.Intent;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.view.ViewStub;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.xxfc.rv.adapter.ChatContentAdapter;
import com.xxfc.rv.audio.IMRecordController;
import com.xxfc.rv.audio.RecordListener;
import com.xxfc.rv.audio_x.VoicePlayer;
import com.xxfc.rv.base.BaseActivity;
import com.xxfc.rv.broadcast.MessageReceiver;
import com.xxfc.rv.db.table.ChatMessage;
import com.xxfc.rv.mvp.presenter.ChatPresenterImp;
import com.xxfc.rv.mvp.presenter.IChatPresenter;
import com.xxfc.rv.mvp.view.IChatView;
import com.xxfc.rv.photopicker.PhotoPickerActivity;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import pub.devrel.easypermissions.AppSettingsDialog;
import pub.devrel.easypermissions.EasyPermissions;

public class ChatActivity extends BaseActivity implements EasyPermissions.PermissionCallbacks, IChatView {
    private TextView tvTitleCenter;
    private ImageView ivTitleLeft;
    private LinearLayout rlChatMeun;
    private FrameLayout rightLayout;
    private ImageButton voiceImgBtn;
    private ImageButton btnCancelReplay;
    private EditText chatEdit;
    private Button recordBtn;
    private LinearLayout leftLayout;
    private ImageButton emotionBtn; // 表情
    private ImageButton moreBtn;
    private Button sendBtn;
    private RecyclerView recycleContent;
    private ViewStub chatToolsViewStub;

    private String mLoginUserId;
    private String chatTargetUserId = "10000028";
    private String mLoginUserName;
    private String toUserName;

    private List<ChatMessage> messages = new ArrayList<>();
    private ChatContentAdapter adapter;
    private MessageReceiver receiver = null;

    private LinearLayout lLMoreSelect;

    private String cameraUrl = null;

    private IMRecordController mRecordController;
    private IChatPresenter presenter;

    @Override
    public int getLayout() {
        return R.layout.activity_chat;
    }

    @Override
    public void initView() {

        mLoginUserId = ImSetting.getUserId();
        mLoginUserName = ImSetting.getUserName();
        presenter = new ChatPresenterImp(this, this);
        presenter.setChatInfo(mLoginUserId, "蒋立新", chatTargetUserId, toUserName);
        presenter.registerImReceiver();

        tvTitleCenter = findViewById(R.id.tv_title_center);
        ivTitleLeft = findViewById(R.id.iv_title_left);
        rlChatMeun = findViewById(R.id.rl_chat_meun);
        rightLayout = findViewById(R.id.right_layout);
        voiceImgBtn = findViewById(R.id.voice_img_btn);
        btnCancelReplay = findViewById(R.id.btnCancelReplay);
        chatEdit = findViewById(R.id.chat_edit);
        recordBtn = findViewById(R.id.record_btn);
        leftLayout = findViewById(R.id.left_layout);
        emotionBtn = findViewById(R.id.emotion_btn);
        moreBtn = findViewById(R.id.more_btn);
        sendBtn = findViewById(R.id.send_btn);
        chatToolsViewStub = findViewById(R.id.chat_tools_view_stub);

        recycleContent = findViewById(R.id.recycle_content);
        recycleContent.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));

        adapter = new ChatContentAdapter(getApplicationContext(), mLoginUserId, messages);
        recycleContent.setAdapter(adapter);

        chatEdit.addTextChangedListener(watcher);
        sendBtn.setOnClickListener(this);
        moreBtn.setOnClickListener(this);
        voiceImgBtn.setOnClickListener(this);
        btnCancelReplay.setOnClickListener(this);
        mRecordController = new IMRecordController(this);
        mRecordController.setRecordListener(new RecordListener() {
            @Override
            public void onRecordSuccess(String filePath, int timeLen) {
                // 录音成功，返回录音文件的路径
                recordBtn.setText(R.string.motalk_voice_chat_tip_1);
                recordBtn.setBackgroundResource(R.drawable.im_voice_button_normal2);
                if (timeLen < 1) {
                    return;
                }
                presenter.sendVoice(filePath, timeLen);
            }

            @Override
            public void onRecordStart() {
                VoicePlayer.instance().stop();
                // 录音开始
                recordBtn.setText(R.string.motalk_voice_chat_tip_2);
                recordBtn.setBackgroundResource(R.drawable.im_voice_button_pressed2);
            }

            @Override
            public void onRecordCancel() {
                // 录音取消
                recordBtn.setText(R.string.motalk_voice_chat_tip_1);
                recordBtn.setBackgroundResource(R.drawable.im_voice_button_normal2);
            }
        });
        recordBtn.setOnTouchListener(mRecordController);
    }

    @Override
    public void getData() {
        List<ChatMessage> lists = ((ChatPresenterImp) presenter).getDbService().getAllMessage(chatTargetUserId);
        if (lists != null) {
            messages.addAll(lists);
            adapter.notifyDataSetChanged();
            if (messages.size() > 0) {
                recycleContent.scrollToPosition(messages.size() - 1);
            }
        }
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.send_btn) { // 发送消息
            if (TextUtils.isEmpty(chatEdit.getText().toString().trim())) {
                Toast.makeText(getApplicationContext(), "发送的消息不能为空", Toast.LENGTH_SHORT).show();
                return;
            }
            presenter.sendText(chatEdit.getText().toString().trim());
            chatEdit.getEditableText().clear();
        } else if (id == R.id.more_btn) {
            changeChatToolsView();
        } else if (id == R.id.tv_pic) {
            isCamera = false;
            presenter.clickPicPicker(isCamera);
        } else if (id == R.id.tv_photo) {
            isCamera = true;
            presenter.clickPicPicker(isCamera);
        } else if (id == R.id.voice_img_btn) {
            chatEdit.setVisibility(View.GONE);
            recordBtn.setVisibility(View.VISIBLE);
            btnCancelReplay.setVisibility(View.VISIBLE);
            voiceImgBtn.setVisibility(View.GONE);
        } else if (id == R.id.btnCancelReplay) {
            chatEdit.setVisibility(View.VISIBLE);
            recordBtn.setVisibility(View.GONE);
            btnCancelReplay.setVisibility(View.GONE);
            voiceImgBtn.setVisibility(View.VISIBLE);
        } else if (id == R.id.record_btn) {

        } else if (id == R.id.tv_file) {

        } else if (id == R.id.tv_vilte) {
            presenter.realDial(1, null);

        } else if (id == R.id.tv_video) {

        }
    }

    private boolean isCamera = false;

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        EasyPermissions.onRequestPermissionsResult(requestCode, permissions, grantResults, this);
    }

    @Override
    public void onPermissionsGranted(int requestCode, @NonNull List<String> perms) {
        presenter.clickPicPicker(isCamera);
    }

    @Override
    public void onPermissionsDenied(int requestCode, @NonNull List<String> perms) {
        Toast.makeText(this, "用户授权失败", Toast.LENGTH_SHORT).show();
        if (EasyPermissions.somePermissionPermanentlyDenied(this, perms)) {
            new AppSettingsDialog.Builder(this).
                    setTitle("需要权限")
                    .setRationale("如果没有这些权限，此应用程序可能无法正常工作,是否打开设置修改该应用权限")
                    .build().show();
        }
    }

    private void changeChatToolsView() {

        if (lLMoreSelect == null) {
            lLMoreSelect = (LinearLayout) chatToolsViewStub.inflate();
            lLMoreSelect.findViewById(R.id.tv_photo).setOnClickListener(this);
            lLMoreSelect.findViewById(R.id.tv_pic).setOnClickListener(this);
            lLMoreSelect.findViewById(R.id.tv_file).setOnClickListener(this);
            lLMoreSelect.findViewById(R.id.tv_vilte).setOnClickListener(this);
            lLMoreSelect.findViewById(R.id.tv_video).setOnClickListener(this);
        }

        boolean isShowing = isToolsShown();
        if (!isShowing) {
            lLMoreSelect.setVisibility(View.VISIBLE);
        } else {
            lLMoreSelect.setVisibility(View.GONE);
        }
    }

    private boolean isToolsShown() {
        return lLMoreSelect.getVisibility() == View.GONE;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (receiver != null) {
            unregisterReceiver(receiver);
            receiver = null;
        }
    }

    private TextWatcher watcher = new TextWatcher() {
        @Override
        public void beforeTextChanged(CharSequence s, int start, int count, int after) {

        }

        @Override
        public void onTextChanged(CharSequence s, int start, int before, int count) {
            if (s == null) {
                sendBtn.setVisibility(View.GONE);
                return;
            }
            if (s.length() == 0) {
                sendBtn.setVisibility(View.GONE);
            } else {
                sendBtn.setVisibility(View.VISIBLE);
            }
        }

        @Override
        public void afterTextChanged(Editable s) {

        }
    };

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == RESULT_OK) {
            switch (requestCode) {
                case REQUEST_CODE_PICK_PHOTO:
                    if (data != null) {
                        boolean isOriginal = data.getBooleanExtra(PhotoPickerActivity.EXTRA_RESULT_ORIGINAL, false);
                        presenter.album(data.getStringArrayListExtra(PhotoPickerActivity.EXTRA_RESULT), isOriginal);
                    } else {
                        Toast.makeText(this, R.string.c_photo_album_failed, Toast.LENGTH_SHORT).show();
                    }
                    break;
                case REQUEST_CODE_CAMERA:
                    if (!TextUtils.isEmpty(getCameraStoreUrl())) {
                        String url = String.format("%1$s%2$s", "file://", getCameraStoreUrl());
//                        UploadEngine.uploadImFile(getApplicationContext(), ImSetting.getImToken(getApplicationContext()), fromUserId, toUserId, message, mUploadResponse);
                    }
                    break;
            }
        }
    }


    private String getCameraStoreUrl() {
        if (TextUtils.isEmpty(cameraUrl) && new File(cameraUrl).exists()) {
            return cameraUrl;
        }
        return "";
    }


    @Override
    public void sendMessSuccess(String receiptId) {
        if (!TextUtils.isEmpty(receiptId) && messages != null && messages.size() > 0) {
            for (int i = messages.size() - 1; i >= 0; i--) {
                ChatMessage chatMessage = messages.get(i);
                if (receiptId.equals(chatMessage.getPackId())) {
                    chatMessage.setMessageStatus(1);
                    adapter.notifyItemChanged(i);
                    break;
                }
            }
        }
    }

    @Override
    public void addMessage(ChatMessage message) {
        messages.add(message);
        adapter.notifyDataSetChanged();
        recycleContent.scrollToPosition(adapter.getItemCount() - 1);
    }

    @Override
    public void upSuccess(String toUserId, ChatMessage message) {
        if (messages.contains(message)) {
            int i = messages.lastIndexOf(message);
            adapter.notifyItemChanged(i);
            presenter.sendMsgBroad(message);
        }
    }

    @Override
    public void upFailure(String toUserId, ChatMessage message) {
        if (messages.contains(message)) {
            int i = messages.lastIndexOf(message);
            adapter.notifyItemChanged(i);
        }
    }

    @Override
    public void upProgress(String toUserId, ChatMessage message) {
        if (messages.contains(message)) {
            int i = messages.lastIndexOf(message);
            adapter.notifyItemChanged(i);
        }
    }

    @Override
    public void receiverMsg(ChatMessage message) {
        if (message != null && !TextUtils.isEmpty(chatTargetUserId) && chatTargetUserId.equals(message.getFromUserId())) {
            messages.add(message);
            adapter.notifyDataSetChanged();
            if(messages.size() >0){
                recycleContent.scrollToPosition(messages.size() - 1);
            }
        }
    }
}
