package com.xxfc.rv;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Binder;
import android.os.IBinder;
import android.text.TextUtils;
import android.util.Log;

import com.google.gson.Gson;
import com.xxfc.rv.broadcast.MessageReceiver;
import com.xxfc.rv.db.MessageTypeEnum;
import com.xxfc.rv.db.table.ChatMessage;
import com.xxfc.rv.listener.SendMessageListener;
import com.xxfc.rv.log.LogUtil;
import com.xxfc.rv.service.ChatMessageServiceImp;
import com.xxfc.rv.util.NetworkUtil;
import com.xxfc.rv.xmpp.NotifyConnectionListener;
import com.xxfc.rv.xmpp.ReceiptManager;
import com.xxfc.rv.xmpp.XChatManager;
import com.xxfc.rv.xmpp.XmppConnectionManager;

import org.jivesoftware.smack.XMPPConnection;

public class ImService extends Service {

    private static final String TAG = ImService.class.getSimpleName();
    private static final Intent SERVICE_INTENT = new Intent();
    private XmppConnectionManager mConnectionManager;// 唯一
    private boolean isInit = true;
    private String mLoginUserId;
    private String mLoginPassword;
    private String mLoginNickName;

    private XChatManager mXChatManager;// 唯一
    private ReceiptManager mReceiptManager;// 唯一

    public String getUserId() {
        return mLoginUserId;
    }

    public static Intent getIntent() {
        return SERVICE_INTENT;
    }

    // 要用ContextCompat.startForegroundService启动，否则安卓8.0以上可能崩溃，而且是不一定复现的那种，
    public static Intent getIntent(Context context, String userId, String password, String nickName) {
        Intent intent = new Intent(context, ImService.class);
        intent.putExtra(Constants.EXTRAS.EXTRA_LOGIN_USER_ID, userId);
        intent.putExtra(Constants.EXTRAS.EXTRA_LOGIN_PASSWORD, password);
        intent.putExtra(Constants.EXTRAS.EXTRA_LOGIN_NICK_NAME, nickName);
        return intent;
    }

    private MessageReceiver receiver = new MessageReceiver(new SendMessageListener() {
        @Override
        public void send(String toUserId, String message) {
            ChatMessage chatMessage = new Gson().fromJson(message, ChatMessage.class);
            sendChatMessage(toUserId, chatMessage);
        }

        @Override
        public void sendMsgSuccess(String receiptId) {

        }

        @Override
        public void receiverMsg(String msg) {

        }
    });

    private NotifyConnectionListener mNotifyConnectionListener = new NotifyConnectionListener() {
        @Override
        public void notifyConnecting() {
            LogUtil.e(TAG, "xmpp:notifyConnecting");
//            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_ING);
        }

        @Override
        public void notifyConnected(XMPPConnection arg0) {
            LogUtil.e(TAG, "xmpp:notifyConnected");
//            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_ING);
        }

        @Override
        public void notifyAuthenticated(XMPPConnection arg0) {
            LogUtil.e(TAG, "xmpp:notifyAuthenticated");
            onAuthenticated();
//            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_SUCCESS);// 通知登陆成功
//            authenticatedOperating();
        }

        @Override
        public void notifyConnectionClosedOnError(Exception arg0) {
            LogUtil.e(TAG, "xmpp:notifyConnectionClosedOnError");
//            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_NOT);
        }

        @Override
        public void notifyConnectionClosed() {
            LogUtil.e(TAG, "xmpp:notifyConnectionClosed");
//            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_NOT);
        }
    };

    private ImServiceBinder mBinder = new ImServiceBinder();

    @Override
    public IBinder onBind(Intent intent) {
        LogUtil.d(TAG, "ImService onBind");
        return mBinder;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        register();

        mLoginUserId = ImSetting.getUserId();
        mLoginPassword = ImSetting.getUserPassword();
        mLoginNickName = ImSetting.getUserName();
        init();
    }

//    @Override
//    public int onStartCommand(Intent intent, int flags, int startId) {
//
//        mLoginUserId = intent.getStringExtra(Constants.EXTRAS.EXTRA_LOGIN_USER_ID);
//        mLoginPassword = intent.getStringExtra(Constants.EXTRAS.EXTRA_LOGIN_PASSWORD);
//        mLoginNickName = intent.getStringExtra(Constants.EXTRAS.EXTRA_LOGIN_NICK_NAME);
//        LogUtil.e(TAG, "ImService onStartCommand");
//        init();
//        return START_STICKY;
//    }

    private void onAuthenticated() {
        if (!isAuthenticated()) {
            return;
        }

        /* 消息回执管理 */
        if (mReceiptManager == null) {
            mReceiptManager = new ReceiptManager(this, mConnectionManager.getConnection());
        } else {
            mReceiptManager.reset();
        }

        // 初始化消息处理
        if (mXChatManager == null) {
            mXChatManager = new XChatManager(this, mConnectionManager.getConnection());
        } else {
            mXChatManager.reset();
        }

        /*  获取离线消息 */
        mConnectionManager.sendOnLineMessage();
    }

    public boolean isAuthenticated() {
        if (mConnectionManager != null && mConnectionManager.isAuthenticated()) {
            return true;
        }
        return false;
    }

    private void init() {

        if (mConnectionManager == null) {
            initConnection();
        }

        if (isInit) {
            Log.e("zq", "isInit==true,直接登录");
            login(mLoginUserId, mLoginPassword);
            return;
        }
        isInit = true;
//        if (Constants.IS_CLOSED_ON_ERROR_END_DOCUMENT && mConnectionManager != null) {
//            Log.e("zq", "CLOSED_ON_ERROR_END_DOCUMENT--->调用release方法");
//            Constants.IS_CLOSED_ON_ERROR_END_DOCUMENT = false;
//            release();
//        }
    }

    public void initConnection() {
        mConnectionManager = new XmppConnectionManager(this, mNotifyConnectionListener);
    }

    public void login(String userId, String password) {
        if (!TextUtils.isEmpty(userId) && !TextUtils.isEmpty(password) && mConnectionManager != null) {
            mConnectionManager.login(userId, password);
        }
    }


    public class ImServiceBinder extends Binder {
        public ImService getService() {
            return ImService.this;
        }
    }

    /******
     * 注册接受消息的广播
     */
    private void register() {
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(Constants.ACTIONS.ACTION_SEND_RECEIVER);
        registerReceiver(receiver, intentFilter);
    }

    private void release() {
        if (mConnectionManager != null) {
            mConnectionManager.release();
            mConnectionManager = null;
        }

        mReceiptManager = null;
        mXChatManager = null;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        release();

        if (receiver != null) {
            unregisterReceiver(receiver);
            receiver = null;
        }
    }

    /**
     * 发送聊天消息
     */
    public void sendChatMessage(String toUserId, ChatMessage chatMessage) {
        if (mXChatManager == null) {
            LogUtil.e(TAG, "mXChatManager==null");
        }

        if (mReceiptManager == null) {
            LogUtil.e(TAG, "mReceiptManager==null");
        }

        if (!isAuthenticated()) {
            Log.e(TAG, "isAuthenticated==false");
        }

        if (mXChatManager == null || mReceiptManager == null
                || (!isAuthenticated() && !NetworkUtil.isNetworkAvailable(ImService.this))) {

        } else {
            /**
             * 先添加一个等待接收回执的消息
             * 然后再发送这条消息
             */
//            mReceiptManager.addWillSendMessage(toUserId, chatMessage, SendType.NORMAL, chatMessage.getContent());
            mXChatManager.sendMessage(toUserId, chatMessage);
        }
    }

    public void notificationMessage(ChatMessage chatMessage) {
        ChatMessageServiceImp.getInstance(getApplicationContext()).addMessage(chatMessage);  // 数据库添加消息
        sendMsgBroad(chatMessage);
    }

    /*****
     * 发送广播，通知界面更新
     * @param chatMessage
     */
    private void sendMsgBroad(ChatMessage chatMessage) {
        Intent intent = new Intent();
        intent.setAction(Constants.ACTIONS.ACTION_RECEIVE_NEW_MESSAGE);
        intent.setPackage(getPackageName());
        intent.putExtra(Constants.KEYS.KEY_NEW_MESSAGE, chatMessage.toJsonString());
        sendBroadcast(intent);
    }


}
