package com.xxfc.rv.db.dao;

import android.content.Context;
import android.text.TextUtils;
import android.util.Log;

import com.j256.ormlite.android.apptools.OpenHelperManager;
import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.DaoManager;
import com.xxfc.rv.db.SQLiteHelper;
import com.xxfc.rv.db.table.ChatMessage;
import com.xxfc.rv.log.LogUtil;

import org.jivesoftware.smack.chat2.Chat;

import java.sql.SQLException;
import java.util.List;

public class MessageDao {

    private static final String TAG = MessageDao.class.getSimpleName();
    private static MessageDao instance = null;
    public Dao<ChatMessage, Integer> dao;
    private SQLiteHelper mHelper;
    private Context mContext;

    private MessageDao(Context context) {
        try {
            this.mContext = context;
            mHelper = OpenHelperManager.getHelper(mContext, SQLiteHelper.class);
            dao = DaoManager.createDao(mHelper.getConnectionSource(), ChatMessage.class);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    public static final MessageDao getInstance(Context context) {
        if (instance == null) {
            synchronized (MessageDao.class) {
                if (instance == null) {
                    instance = new MessageDao(context);
                }
            }
        }
        return instance;
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        OpenHelperManager.releaseHelper();
    }


    /*******
     * 插入消息
     * @param message
     */
    public void addMessage(ChatMessage message) {
        if (dao == null) {
            LogUtil.e(TAG, "dao is null");
            return;
        }
        if (message == null) {
            LogUtil.e(TAG, "message is null");
            return;
        }
        Log.d(TAG, message.toString());
        if (isExit(message)) {
            updateMessage(message);
            return;
        }
        try {
            int result = dao.create(message);
            LogUtil.d(TAG, "插入结果 " + result);
        } catch (SQLException e) {
            e.printStackTrace();
            LogUtil.d(TAG, "插入失败 " + e.getMessage());
        }
    }

    /*****
     * 更新消息
     * @param message
     */
    public void updateMessage(ChatMessage message) {
        if (dao == null) {
            LogUtil.e(TAG, "dao is null");
            return;
        }
        if (message == null) {
            LogUtil.e(TAG, "message is null");
            return;
        }
        try {
            dao.update(message);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    /*******
     * 判断是否存在该消息
     * @param message
     * @return
     */
    public boolean isExit(ChatMessage message) {
        if (dao == null) {
            LogUtil.e(TAG, "dao is null");
            return false;
        }
        if (message == null) {
            LogUtil.e(TAG, "message is null");
            return false;
        }
        try {
            List<ChatMessage> lists = dao.queryForEq("packId", message.getPackId());
            return lists == null ? false : lists.size() == 0 ? false : true;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return false;
    }

    /******
     * 根据对方的消息id 获取所有聊天
     * @param targetId
     * @return
     */
    public List<ChatMessage> getAllChatMessageByConversation(String targetId) {
        if (dao == null) {
            LogUtil.e(TAG, "dao is null");
            return null;
        }
        if (TextUtils.isEmpty(targetId)) {
            LogUtil.e(TAG, "message is null");
            return null;
        }
        List<ChatMessage> chatMessages = null;
        try {
            chatMessages = dao.queryBuilder().orderBy("timeReceive", false).where().
                    eq("fromUserId", targetId)
                    .or()
                    .eq("toUserId", targetId)
                    .query();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return chatMessages;
    }

    public void setMsgSuccess(String receiptId) {
        try {
            List<ChatMessage> chatMessages = dao.queryBuilder().orderBy("timeSend", false).where().eq("packId", receiptId).query();
            if (chatMessages != null && chatMessages.size() > 0) {
                ChatMessage message = chatMessages.get(0);
                message.setMessageStatus(1);
                updateMessage(message);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    public void setUpProgress(String packId, int progress) {
        try {
            List<ChatMessage> chatMessages = dao.queryBuilder().orderBy("timeSend", false).where().eq("packId", packId).query();
            if (chatMessages != null && chatMessages.size() > 0) {
                ChatMessage message = chatMessages.get(0);
                message.setUploadSchedule(progress);
                if (progress == 100) {
                    message.setUpLoadStatus(2);
                }
                updateMessage(message);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    public void setUpFailure(String packId){
        try {
            List<ChatMessage> chatMessages = dao.queryBuilder().orderBy("timeSend", false).where().eq("packId", packId).query();
            if (chatMessages != null && chatMessages.size() > 0) {
                ChatMessage message = chatMessages.get(0);
                message.setUpLoadStatus(3);
                updateMessage(message);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}
