package com.xxfc.rv.mvp.presenter;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.v4.content.FileProvider;
import android.text.TextUtils;
import android.util.Log;

import com.xxfc.rv.Constants;
import com.xxfc.rv.ImSetting;
import com.xxfc.rv.broadcast.MessageReceiver;
import com.xxfc.rv.call.Jitsi_pre;
import com.xxfc.rv.db.CallTypeEnum;
import com.xxfc.rv.db.MessageTypeEnum;
import com.xxfc.rv.db.table.ChatMessage;
import com.xxfc.rv.listener.SendMessageListener;
import com.xxfc.rv.mvp.view.IChatView;
import com.xxfc.rv.photopicker.SelectModel;
import com.xxfc.rv.photopicker.intent.PhotoPickerIntent;
import com.xxfc.rv.service.ChatMessageServiceImp;
import com.xxfc.rv.service.IChatMessageService;
import com.xxfc.rv.util.UploadEngine;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import pub.devrel.easypermissions.EasyPermissions;
import top.zibin.luban.Luban;
import top.zibin.luban.OnCompressListener;

public class ChatPresenterImp implements IChatPresenter {

    private WeakReference<Context> weakReference = null;

    private IChatView chatView = null;

    private MessageReceiver receiver = null;

    private String fromUserId;
    private String toUserId = "10000062";
    private String fromUserName;
    private String toUserName;

    public ChatPresenterImp(Context context, IChatView chatView) {
        this.weakReference = new WeakReference<>(context);
        this.chatView = chatView;
    }


    @Override
    public void setChatInfo(String fromUserId, String fromUserName, String toUserId, String toUserName) {
        this.fromUserId = fromUserId;
        this.toUserId = toUserId;
        this.fromUserName = fromUserName;
        this.toUserName = toUserName;
    }

    @Override
    public void sendVoice(String filePath, int timeLen) {

        if (TextUtils.isEmpty(filePath)) {
            return;
        }
        ChatMessage message = new ChatMessage();
        message.setMessageType(MessageTypeEnum.VOICE);
        message.setUpLoadStatus(1);
        message.setUploadSchedule(0);
        message.setContent("");
        message.setFilePath(filePath);
        message.setTimeLen(timeLen);
        sendMessage(message);
    }

    @Override
    public void clickPicPicker(boolean isCamera) {
        if (!EasyPermissions.hasPermissions(weakReference.get(), cameraPermissions)) {
            EasyPermissions.requestPermissions((Activity) weakReference.get(), "拍照需要此权限，否则无法正常使用", 1, cameraPermissions);
            return;
        }
        if (isCamera) {
            clickCamera();
        } else {
            openAlbum();
        }
    }

    @Override
    public void openAlbum() {
        ArrayList<String> imagePaths = new ArrayList<>();
        PhotoPickerIntent intent = new PhotoPickerIntent(weakReference.get());
        intent.setSelectModel(SelectModel.MULTI);
        // 已选中的照片地址， 用于回显选中状态
        intent.setSelectedPaths(imagePaths);
        ((Activity) weakReference.get()).startActivityForResult(intent, IChatView.REQUEST_CODE_PICK_PHOTO);
    }

    private String[] cameraPermissions = {Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA, Manifest.permission.READ_EXTERNAL_STORAGE};

    @Override
    public void clickCamera() {

        File cameraSavePath = new File(Environment.getExternalStorageDirectory().getPath() + "/" + System.currentTimeMillis() + ".jpg");
        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        Uri uri = null;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            //第二个参数为 包名.fileprovider
            uri = FileProvider.getUriForFile(weakReference.get(), "com.xxfc.rv.fileprovider", cameraSavePath);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        } else {
            uri = Uri.fromFile(cameraSavePath);
        }
        intent.putExtra(MediaStore.EXTRA_OUTPUT, uri);
        ((Activity) weakReference.get()).startActivityForResult(intent, IChatView.REQUEST_CODE_CAMERA);
    }

    @Override
    public void sendText(String text) {
        if (TextUtils.isEmpty(text)) {
            return;
        }
        ChatMessage message = new ChatMessage();
        message.setMessageType(MessageTypeEnum.TEXT);
        message.setFromUserId(fromUserId);
        message.setFromUserName(fromUserName);
        message.setContent(text);
        sendMessage(message);
    }

    @Override
    public void realDial(int type, String meetUrl) {
        ChatMessage message = new ChatMessage();
        if (type == 1) {// 语音通话
            message.setMessageType(MessageTypeEnum.VOLTE_CALL);
            message.setContent("邀请您 语音通话");
        } else {// 视频通话
            message.setMessageType(MessageTypeEnum.VIDEO_CALL);
            message.setContent("邀请您 视频通话");
        }
        message.setFromUserId(fromUserId);
        message.setFromUserName(fromUserName);
        message.setToUserId(toUserId);
        message.setCallType(CallTypeEnum.CONNECT_VOICE);
        if (!TextUtils.isEmpty(meetUrl)) {
            message.setFilePath(meetUrl);
        }
        message.setPackId(UUID.randomUUID().toString().replaceAll("-", ""));
        message.setTimeSend((int) System.currentTimeMillis() / 1000);
        sendMsgBroad(message);
        Intent intent = new Intent(weakReference.get(), Jitsi_pre.class);
        if (type == 1) {
            intent.putExtra("isvoice", true);
        } else {
            intent.putExtra("isvoice", false);
        }
        intent.putExtra("fromuserid", fromUserId);
        intent.putExtra("touserid", toUserId);
        intent.putExtra("username", toUserName);
        if (!TextUtils.isEmpty(meetUrl)) {
            intent.putExtra("meetUrl", meetUrl);
        }
        weakReference.get().startActivity(intent);
    }

    @Override
    public void sendMessage(ChatMessage message) {

        message.setFromUserId(fromUserId);
        message.setFromUserName(fromUserName);
        message.setToUserName(toUserName);
        message.setFromId("android");
        message.setToUserId(toUserId);
        message.setPackId(UUID.randomUUID().toString().replaceAll("-", ""));
        message.setTimeSend((int) (System.currentTimeMillis() / 1000));
        message.setMessageStatus(0);
        getDbService().addMessage(message);

        chatView.addMessage(message);
        if (message.getMessageType() == MessageTypeEnum.PIC.getType() || message.getMessageType() == MessageTypeEnum.FILE.getType()
                || message.getMessageType() == MessageTypeEnum.VOICE.getType()) {
            if (message.getUpLoadStatus() == 1 || message.getUpLoadStatus() == 3) {
                UploadEngine.uploadImFile(weakReference.get(), ImSetting.getImToken(), fromUserId, toUserId, message, mUploadResponse);
            } else {
                sendMsgBroad(message);
            }
        } else {
            sendMsgBroad(message);
        }
    }

    @Override
    public void sendMsgBroad(ChatMessage message) {
        Intent intent = new Intent();
        intent.setAction(Constants.ACTIONS.ACTION_SEND_RECEIVER);
        intent.setPackage(weakReference.get().getPackageName());
        intent.putExtra(Constants.KEYS.KEY_MESSAGE_TARGET, toUserId);
        intent.putExtra(Constants.KEYS.KEY_MESSAGE_CONTENT, message.toJsonString());
        weakReference.get().sendBroadcast(intent);
    }

    @Override
    public void album(ArrayList<String> stringArrayListExtra, boolean isOriginal) {
        if (isOriginal) {// 原图发送，不压缩
            Log.e("zq", "原图发送，不压缩，开始发送");
            for (int i = 0; i < stringArrayListExtra.size(); i++) {
                sendImage(new File(stringArrayListExtra.get(i)));
            }
            Log.e("zq", "原图发送，不压缩，发送结束");
            return;
        }

        List<File> fileList = new ArrayList<>();
        for (int i = 0; i < stringArrayListExtra.size(); i++) {
            // gif动图不压缩，
            if (stringArrayListExtra.get(i).endsWith("gif")) {
                fileList.add(new File(stringArrayListExtra.get(i)));
                stringArrayListExtra.remove(i);
            } else {
                // Luban只处理特定后缀的图片，不满足的不处理也不走回调，
                // 只能挑出来不压缩，
                List<String> lubanSupportFormatList = Arrays.asList("jpg", "jpeg", "png", "webp", "gif");
                boolean support = false;
                for (int j = 0; j < lubanSupportFormatList.size(); j++) {
                    if (stringArrayListExtra.get(i).endsWith(lubanSupportFormatList.get(j))) {
                        support = true;
                        break;
                    }
                }
                if (!support) {
                    fileList.add(new File(stringArrayListExtra.get(i)));
                    stringArrayListExtra.remove(i);
                }
            }
        }

        if (fileList.size() > 0) {
            for (File file : fileList) {// 不压缩的部分，直接发送
                sendImage(file);
            }
        }

        Luban.with(weakReference.get())
                .load(stringArrayListExtra)
                .ignoreBy(100)// 原图小于100kb 不压缩
                .setCompressListener(new OnCompressListener() {
                    @Override
                    public void onStart() {
                        Log.e("zq", "开始压缩");
                    }

                    @Override
                    public void onSuccess(File file) {
                        sendImage(file);
                    }

                    @Override
                    public void onError(Throwable e) {

                    }
                }).launch();// 启动压缩
    }

    public IChatMessageService getDbService() {
        return ChatMessageServiceImp.getInstance(weakReference.get());
    }

    @Override
    public void sendImage(File file) {

        if (!file.exists()) {
            return;
        }
//        if (isAuthenticated()) {
//            return;
//        }
//        long fileSize = file.length();
        ChatMessage message = new ChatMessage();
        message.setMessageType(MessageTypeEnum.PIC);
        message.setUpLoadStatus(1);
        message.setUploadSchedule(0);
        message.setContent("");
        String filePath = file.getAbsolutePath();
        message.setFilePath(filePath);
        message.setTimeSend((int) (System.currentTimeMillis() / 1000));
//        message.setF((int) fileSize);
//        int[] imageParam = FileDataHelper.getImageParamByIntsFile(filePath);
//        message.setLocation_x(String.valueOf(imageParam[0]));
//        message.setLocation_y(String.valueOf(imageParam[1]));
//        message.setIsReadDel(isReadDel);
//        mChatMessages.add(message);
//        mChatContentView.notifyDataSetInvalidated(true);

        sendMessage(message);
    }

    @Override
    public void registerImReceiver() {
        if (receiver == null) receiver = new MessageReceiver(listener);
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(Constants.ACTIONS.ACTION_SEND_MSG_SUCCESS);
        intentFilter.addAction(Constants.ACTIONS.ACTION_RECEIVE_NEW_MESSAGE);
        weakReference.get().registerReceiver(receiver, intentFilter);
    }


    private SendMessageListener listener = new SendMessageListener() {
        @Override
        public void send(String toUserId, String message) {

        }

        @Override
        public void sendMsgSuccess(String receiptId) {
            chatView.sendMessSuccess(receiptId);
        }

        @Override
        public void receiverMsg(String message) {
            if (!TextUtils.isEmpty(message)) {
                ChatMessage chatMessage = ChatMessage.json2Message(message);
                chatView.receiverMsg(chatMessage);
            }
        }
    };


    private UploadEngine.ImFileUploadResponse mUploadResponse = new UploadEngine.ImFileUploadResponse() {

        @Override
        public void onSuccess(String toUserId, ChatMessage message) {
            chatView.upSuccess(toUserId, message);
        }

        @Override
        public void onFailure(String toUserId, ChatMessage message) {
            chatView.upFailure(toUserId, message);
        }

        @Override
        public void onUpProgress(String toUserId, ChatMessage message) {
            chatView.upProgress(toUserId, message);
        }
    };
}
