package com.xxfc.rv.xmpp;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Handler;

import com.xxfc.rv.Constants;
import com.xxfc.rv.ImService;
import com.xxfc.rv.db.table.ChatMessage;
import com.xxfc.rv.log.LogUtil;
import com.xxfc.rv.service.ChatMessageServiceImp;
import com.xxfc.rv.service.IChatMessageService;

import org.jivesoftware.smack.packet.Stanza;
import org.jivesoftware.smack.tcp.XMPPTCPConnection;
import org.jivesoftware.smackx.receipts.DeliveryReceiptManager;
import org.jivesoftware.smackx.receipts.ReceiptReceivedListener;
import org.jxmpp.jid.Jid;

import java.util.HashMap;
import java.util.Map;

public class ReceiptManager {
    public static final int MESSAGE_DELAY = 20 * 1000; // 等待消息回执， 超时时间
    private static final int RECEIPT_NO = 0x1; // 没有收到回执
    private static final int RECEIPT_YES = 0x2;// 收到回执
    /**
     * 处理消息回执
     */
    public static Map<String, ReceiptObj> mReceiptMap = new HashMap<String, ReceiptObj>();
    private ImService mService;
    private XMPPTCPConnection mConnection;
    private String mLoginUserId;// 用于切换用户后，判断是否清除回执内容
    /**
     * 重发次数表
     */
    private Map<String, Integer> mReSendMap = new HashMap<String, Integer>();

    ReceiptReceivedListener mReceiptReceivedListener = (fromJid, toJid, receiptId, receipt) -> {
        LogUtil.e("msg", "收到消息回执:fromJid=" + fromJid + "-----toJid=" + toJid + "-----receiptId=" + receiptId);
        /*************
         * 收到消息回执
         *  1 修改消息的状态为发送成功
         *   2 修改ui显示
         */
        getDbService().setMsgSendSuccess(receiptId);
        sendMsgBroad(receiptId);
    };

    private IChatMessageService getDbService() {
        return ChatMessageServiceImp.getInstance(mService.getApplicationContext());
    }

    private void sendMsgBroad(String receiptId) {
        Intent intent = new Intent();
        intent.setAction(Constants.ACTIONS.ACTION_SEND_MSG_SUCCESS);
        intent.setPackage(mService.getPackageName());
        intent.putExtra(Constants.KEYS.KEY_USER_RECEIPTID,receiptId);
        mService.sendBroadcast(intent);
    }


    public ReceiptManager(ImService service, XMPPTCPConnection connection) {
        mService = service;
        mConnection = connection;
        mLoginUserId = service.getUserId();

        DeliveryReceiptManager mDeliveryReceiptManager = DeliveryReceiptManager.getInstanceFor(mConnection);
        // 自动发送消息回执
        mDeliveryReceiptManager.setAutoReceiptMode(DeliveryReceiptManager.AutoReceiptMode.always);
        // 添加消息回执监听
        mDeliveryReceiptManager.addReceiptReceivedListener(mReceiptReceivedListener);
    }

    public void reset() {
        String userId = mService.getUserId();
        if (!mLoginUserId.equals(userId)) {
            mLoginUserId = userId;
            mReceiptMap.clear();
        }
    }

    /**
     * 发送的消息类型，用于类型判断和消息回执的分发
     */
    public enum SendType {
        NORMAL, PUSH_NEW_FRIEND
    }

    class ReceiptObj {
        String toUserId;    // 普通消息和新朋友消息公用
        ChatMessage msg;    // 用于普通消息和新朋友消息公用
        SendType sendType;  // 用于分发普通消息和新朋友消息的回执
        int Read;           // 用于标记此消息是否为已读回执消息
        String Read_msg_pid;// 被回执消息的packetId
    }
}
